@@ -19,6 +19,8 @@
 // Copyright (C) 2012 Suzuki Toshiya <mpsuzuki@hiroshima-u.ac.jp>
 // Copyright (C) 2016 William Bader <williambader@hotmail.com>
 // Copyright (C) 2018 Adam Reichold <adam.reichold@t-online.de>
+// Copyright (C) 2022 Oliver Sander <oliver.sander@tu-dresden.de>
+// Copyright (C) 2024 g10 Code GmbH, Author: Sune Stolborg Vuorela <sune@vuorela.dk>
 //
 // To see a description of the changes please see the Changelog file that
 // came with your tarball or type make ChangeLog if you are building from git
@@ -30,8 +32,10 @@
 
 #include <cstddef>
 #include <memory>
+#include <vector>
 #include <unordered_map>
 #include <string>
+#include <span>
 #include "FoFiBase.h"
 #include "poppler_private_export.h"
 
@@ -47,7 +51,7 @@ class POPPLER_PRIVATE_EXPORT FoFiTrueType : public FoFiBase
 {
 public:
     // Create a FoFiTrueType object from a memory buffer.
-    static std::unique_ptr<FoFiTrueType> make(const char *fileA, int lenA, int faceIndexA = 0);
+    static std::unique_ptr<FoFiTrueType> make(const unsigned char *fileA, int lenA, int faceIndexA = 0);
 
     // Create a FoFiTrueType object from a file on disk.
     static std::unique_ptr<FoFiTrueType> load(const char *fileName, int faceIndexA = 0);
@@ -155,12 +159,12 @@ public:
     int setupGSUB(const char *scriptName, const char *languageName);
 
 private:
-    FoFiTrueType(const char *fileA, int lenA, bool freeFileDataA, int faceIndexA);
+    FoFiTrueType(const unsigned char *fileA, int lenA, bool freeFileDataA, int faceIndexA);
     void cvtEncoding(char **encoding, FoFiOutputFunc outputFunc, void *outputStream) const;
     void cvtCharStrings(char **encoding, const int *codeToGID, FoFiOutputFunc outputFunc, void *outputStream) const;
     void cvtSfnts(FoFiOutputFunc outputFunc, void *outputStream, const GooString *name, bool needVerticalMetrics, int *maxUsedGlyph) const;
-    void dumpString(const unsigned char *s, int length, FoFiOutputFunc outputFunc, void *outputStream) const;
-    unsigned int computeTableChecksum(const unsigned char *data, int length) const;
+    static void dumpString(std::span<const unsigned char> s, FoFiOutputFunc outputFunc, void *outputStream);
+    static unsigned int computeTableChecksum(std::span<const unsigned char> data);
     void parse();
     void readPostTable();
     int seekTable(const char *tag) const;
@@ -170,10 +174,8 @@ private:
     unsigned int scanLookupSubTable(unsigned int subTable, unsigned int orgGID);
     int checkGIDInCoverage(unsigned int coverage, unsigned int orgGID);
 
-    TrueTypeTable *tables;
-    int nTables;
-    TrueTypeCmap *cmaps;
-    int nCmaps;
+    std::vector<TrueTypeTable> tables;
+    std::vector<TrueTypeCmap> cmaps;
     int nGlyphs;
     int locaFmt;
     int bbox[4];
